@@ -164,6 +164,8 @@ EventMachine_t::~EventMachine_t()
 {
 	// Run down descriptors
 	size_t i;
+	for (i = 0; i < DescriptorsToDelete.size(); i++)
+ 		delete DescriptorsToDelete[i];
 	for (i = 0; i < NewDescriptors.size(); i++)
 		delete NewDescriptors[i];
 	for (i = 0; i < Descriptors.size(); i++)
@@ -840,6 +842,17 @@ void EventMachine_t::_CleanupSockets()
 		EventableDescriptor *ed = Descriptors[i];
 		assert (ed);
 		if (ed->ShouldDelete()) {
+			DescriptorsToDelete.push_back(ed);
+		}
+		else
+			Descriptors [j++] = ed;
+	}
+	while ((size_t)j < Descriptors.size())
+		Descriptors.pop_back();
+
+	nSockets = DescriptorsToDelete.size();
+	for (i=0; i < nSockets; i++) {
+		EventableDescriptor *ed = DescriptorsToDelete[i];
 		#ifdef HAVE_EPOLL
 			if (Poller == Poller_Epoll) {
 				assert (epfd != -1);
@@ -855,13 +868,9 @@ void EventMachine_t::_CleanupSockets()
 				ModifiedDescriptors.erase(ed);
 			}
 		#endif
-			delete ed;
-		}
-		else
-			Descriptors [j++] = ed;
+		delete ed;
 	}
-	while ((size_t)j < Descriptors.size())
-		Descriptors.pop_back();
+	DescriptorsToDelete.clear();
 }
 
 /*********************************
@@ -1142,7 +1151,7 @@ void EventMachine_t::_RunTimers()
 EventMachine_t::InstallOneshotTimer
 ***********************************/
 
-const uintptr_t EventMachine_t::InstallOneshotTimer (int milliseconds)
+const uintptr_t EventMachine_t::InstallOneshotTimer (uint64_t milliseconds)
 {
 	if (Timers.size() > MaxOutstandingTimers)
 		return false;
